@@ -1,8 +1,8 @@
 #!/usr/bin/perl
 #
-# MiniVend version 3.12
+# MiniVend version 2.03
 #
-# $Id: Cart.pm,v 1.14 1999/06/07 08:03:57 mike Exp $
+# $Id: Cart.pm,v 1.6 1997/06/17 04:22:52 mike Exp $
 #
 # This program is largely based on Vend 0.2
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
@@ -11,7 +11,7 @@
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
 #
 # Enhancements made by and
-# Copyright 1996-1999 by Michael J. Heins <mikeh@iac.net>
+# Copyright 1996 by Michael J. Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,23 +30,37 @@
 package Vend::Cart;
 require Exporter;
 
+# AUTOLOAD
+#use AutoLoader;
+#@ISA = qw(Exporter Autoloader);
+#*AUTOLOAD = \&AutoLoader::AUTOLOAD;
+# END AUTOLOAD
+
+# NOAUTO
 @ISA = qw(Exporter);
+# END NOAUTO
 
 @EXPORT = qw(get_cart set_cart toss_cart);
 @EXPORT_OK = qw(create add set);
 
 
-$VERSION = substr(q$Revision: 1.14 $, 10);
+$VERSION = substr(q$Revision: 1.6 $, 10);
 $DEBUG   = 0;
 
+use Carp;
 use vars qw($DEBUG $VERSION);
 use strict;
 
+# AUTOLOAD
+#1;
+#__END__
+# END AUTOLOAD
+
 sub create {
     my($name, @attributes) = @_;
-	die "New shopping cart $name defined with no name.\n"
+	croak "New shopping cart $name defined with no name.\n"
 		unless defined $name;
-	return $Vend::Session->{'carts'}{$name} = [];
+	return $Vend::Session->{'carts'}->{$name} = [];
 }
 
 sub add {
@@ -55,7 +69,7 @@ sub add {
 	my $i;
 	
 	! defined $item{'code'} or $item{'code'} eq ''
-		and die "Vend::Cart: add empty item?";
+		and croak "Vend::Cart: add empty item?";
 
 	$item{'quantity'} ||= 1;
 	$item{'quantity'} < 1 and die "Can't order negative number.\n";
@@ -119,14 +133,11 @@ sub set {
 sub toss_cart {
 	my($s) = @_;
 	my $i;
-	my (@master);
     DELETE: for (;;) {
         foreach $i (0 .. $#$s) {
-            if ($s->[$i]->{'quantity'} <= 0) {
+            if ($s->[$i]->{'quantity'} == 0) {
 				next if defined $s->[$i]->{mv_control} and
 								$s->[$i]->{mv_control} =~ /\bnotoss\b/;
-				push (@master, $s->[$i]->{mv_mi})
-					if $s->[$i]->{mv_mi} && ! $s->[$i]->{mv_si};
                 splice(@$s, $i, 1);
                 next DELETE;
             }
@@ -134,33 +145,20 @@ sub toss_cart {
         last DELETE;
     }
 
-	return 1 unless @master;
-	my $mi;
-	my %save;
-	my @items;
-	# Brute force delete for subitems of any deleted master items
-	foreach $mi (@master) {
-        foreach $i (0 .. $#$s) {
-            $save{$i} = 1
-				unless $s->[$i]->{'mv_si'} and $s->[$i]->{'mv_mi'} eq $mi;
-        }
-	}
-	@items = @$s;
-	@{$s} = @items[sort {$a <=> $b} keys %save];
     1;
 }
 
 sub set_cart {
 	my($cart) = @_;
-	defined $cart and defined $Vend::Session->{'carts'}{$cart}
-	    and return $Vend::Items = $Vend::Session->{'carts'}{$cart};
+	defined $cart and defined $Vend::Session->{'carts'}->{$cart}
+	    and return $Vend::Items = $Vend::Session->{'carts'}->{$cart};
 	return $Vend::Items;
 }
 
 sub get_cart {
 	my($cart,@options) = @_;
-	if(defined $cart and defined $Vend::Session->{'carts'}{$cart}) {
-	    return $Vend::Session->{'carts'}{$cart};
+	if(defined $cart and defined $Vend::Session->{'carts'}->{$cart}) {
+	    return $Vend::Session->{'carts'}->{$cart};
 	}
 	elsif(defined $cart and $cart) {
 		return create $cart;
