@@ -1,10 +1,10 @@
 # Vend/Glimpse.pm:  Search indexes with Glimpse
 #
-# $Id: Glimpse.pm,v 1.20 1999/06/07 08:06:32 mike Exp mike $
+# $Id: Glimpse.pm,v 1.6 1997/06/17 04:22:52 mike Exp $
 #
 # ADAPTED FOR USE WITH MINIVEND from Search::Glimpse
 #
-# Copyright 1996-1999 by Michael J. Heins <mikeh@iac.net>
+# Copyright 1996 by Michael J. Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,13 +20,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
+##  vi regex to delete debugs     %s/^\(.*$s->debug\)/#\1/
+##  vi regex to restore debugs     %s/^\([^#]*\)#\(.*$s->debug\)/\1\2/
 #
 
 package Vend::Glimpse;
 require Vend::Search;
 @ISA = qw(Vend::Search);
 
-$VERSION = substr(q$Revision: 1.20 $, 10);
+$VERSION = substr(q$Revision: 1.6 $, 10);
+use Text::ParseWords;
 use strict;
 
 sub new {
@@ -56,6 +59,18 @@ sub version {
 	$Vend::Glimpse::VERSION;
 }
 
+sub quoted_string {
+
+my ($s, $text) = @_;
+my (@fields);
+push(@fields, $+) while $text =~ m{
+   "([^\"\\]*(?:\\.[^\"\\]*)*)"\s?  ## standard quoted string, w/ possible comma
+   | ([^\s]+)\s?                    ## anything else, w/ possible comma
+   | \s+                            ## any whitespace
+	    }gx;
+   return grep /\w/, @fields;
+}
+
 sub escape {
 	my($self, @text) = @_;
 	if($self->{'global'}->{all_chars}) {
@@ -74,7 +89,7 @@ sub search {
 	my($delim,$string);
 	my($max_matches,$mod,$index_delim,$return_delim);
 	my($cmd,$code,$count,$joiner,$matches_to_send,@out);
-	my($limit_sub,$return_sub,$delayed_return,$prospect);
+	my($limit_sub,$return_sub);
 	my($f,$key,$spec,$val,$range_op);
 	my($searchfile,@searchfiles);
 	my(@pats);
@@ -87,11 +102,7 @@ sub search {
     $g->{return_delim} = $index_delim
 	        unless defined $g->{return_delim};
 
-# DEBUG
-#Vend::Util::logDebug
-#($s->dump_options())
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug($s->dump_options());
 
 	$g->{matches} = 0;
 	$max_matches = $g->{max_matches};
@@ -146,8 +157,7 @@ sub search {
     # Build glimpse line
     push @cmd, $g->{glimpse_cmd};
     unless (defined $g->{search_server}) {
-    	push @cmd, "-H $g->{base_directory}"
-			unless $g->{glimpse_cmd} =~ /\s+-H/;
+    	push @cmd, "-H $g->{base_directory}";
 	}
 	else {
     	push @cmd, "-C $g->{search_server}";
@@ -160,9 +170,6 @@ sub search {
         push @cmd, '-' . $g->{spelling_errors};
     }
 
-	$g->{search_file} = $g->{search_file}[0] 
-		if ref $g->{search_file};
-
     push @cmd, "-i" unless $g->{case_sensitive};
     push @cmd, "-h" unless $g->{return_file_name};
     push @cmd, "-y -L $max_matches:0:$max_matches";
@@ -196,35 +203,30 @@ sub search {
 		push @cmd, "-d '$g->{record_delim}'";
 	}
 
+	$g->{coordinate} = '';
+
     if (! $g->{exact_match}) {
-        @specs = Text::ParseWords::shellwords( join ' ', @specs);
+        @specs = $s->quoted_string( join ' ', @specs);
     }
 
-# DEBUG
-#Vend::Util::logDebug
-#("spec='" . (join "','", @specs) . "'\n")
-#	if ::debug(0x10);
-# END DEBUG
+    @specs = $s->escape(@specs);
 
-	@pats = $s->spec_check($g, @specs);
+#    $s->debug("spec='" . (join "','", @specs) . "'");
 
-	if($g->{regex_specs}) {
-		@pats = @{$g->{regex_specs}};
-	}
+    # untaint
+    for(@specs) {
+        /(.*)/;
+        push @pats, $1;
+    }
+	@{$s->{'specs'}} = @specs;
 
-# DEBUG
-#Vend::Util::logDebug
-#("pats: '" .  join("', '", @pats) . "'")
-#	if ::debug(0x10);
-#Vend::Util::logDebug
-#($s->dump_options())
-#	if ::debug(0x10);
-# END DEBUG
+#    $s->debug("pats: '", join("', '", @pats), "'");
+
+#	$s->debug($s->dump_options());
 
 	$joiner = $g->{or_search} ? ',' : ';';
   CREATE_LIMIT: {
   	$f = sub { 1; };
-  	last CREATE_LIMIT if $g->{spelling_errors};
   	last CREATE_LIMIT unless scalar @{$s->{'fields'}};
   	last CREATE_LIMIT if $g->{'coordinate'};
 	if ($g->{or_search}) {
@@ -246,8 +248,7 @@ sub search {
 	}
   } # last CREATE_LIMIT:
 
-	eval {($limit_sub,$prospect) = $s->get_limit($f)}
-		unless $g->{return_file_name};
+	eval {$limit_sub = $s->get_limit($f)};
 	if($@) {
 		&{$g->{error_routine}}($g->{error_page}, $@);
 		$g->{matches} = -1;
@@ -256,19 +257,11 @@ sub search {
 
 	$spec = join $joiner, @pats;
     push @cmd, "'$spec'";
-# DEBUG
-#Vend::Util::logDebug
-#	("spec: '$spec'")
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug("spec: '", $spec, "'");
 
 	$joiner = $spec;
 	$joiner =~ s/['";,]//g;
-# DEBUG
-#Vend::Util::logDebug
-#("joiner: '$spec'\n")
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug("joiner: '", $spec, "'");
 	if(length($joiner) < $g->{min_string}) {
 		my $msg = <<EOF;
 Search strings must be at least $g->{min_string} characters.
@@ -281,11 +274,7 @@ EOF
 
     $cmd = join ' ', @cmd;
 
-# DEBUG
-#Vend::Util::logDebug
-# ("Glimpse command line: $cmd\n")
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug("Glimpse command line: $cmd");
 
     # searches for debug
     $Vend::Glimpse::PID = open(Vend::Glimpse::SEARCH,qq!$cmd |$sort_string !);
@@ -303,15 +292,11 @@ EOF
 
 
 	if($g->{return_file_name}) {
-# DEBUG
-#Vend::Util::logDebug
-#("Got to return_fields FILENAME\n")
-#	if ::debug(0x10);
-# END DEBUG
+#		$s->debug("Got to return_fields FILENAME");
 		$return_sub = sub {@_};
 	}
 	else {
-		eval {($return_sub, $delayed_return) = $s->get_return()};
+		eval {$return_sub = $s->get_return()};
 		if($@) {
 			&{$g->{error_routine}}($g->{error_page}, $@);
 			$g->{matches} = -1;
@@ -319,11 +304,7 @@ EOF
 		}
 	}
 
-# DEBUG
-#Vend::Util::logDebug
-#('fields/specs: ' .  scalar @{$s->{fields}} .  "/" . scalar @{$s->{specs}} . "\n")
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug('fields/specs: ', scalar @{$s->{fields}}, " ", scalar @{$s->{specs}});
 
 	local($/) = $g->{record_delim};
 
@@ -362,38 +343,18 @@ EOF
 		return undef;
 	}
 
-	if($delayed_return) {
-		$s->sort_search_return(\@out);
-		@out = map { &{$delayed_return}($_) } @out;
-	}
-
-	if($g->{unique_result}) {
-		my %seen;
-		@out = grep ! $seen{$_}++, @out;
-	}
-
 	$g->{matches} = scalar(@out);
+	$g->{first_match} = 0;
+
 
     if ($g->{matches} > $g->{match_limit}) {
         $s->save_more(\@out)
             or &{$g->{log_routine}}("Error saving matches: $!");
-		if ($g->{first_match}) {
-			splice(@out,0,$g->{first_match}) if $g->{first_match};
-			$g->{next_pointer} = $g->{first_match} + $g->{match_limit};
-			$g->{next_pointer} = 0
-				if $g->{next_pointer} > $g->{matches};
-		}
         $#out = $g->{match_limit} - 1;
     }
 
-# DEBUG
-#Vend::Util::logDebug
-#("$g->{matches} matches\n")
-#	if ::debug(0x10);
-#Vend::Util::logDebug
-#("0 .. " . (scalar(@out) - 1) . "\n")
-#	if ::debug(0x10);
-# END DEBUG
+#	$s->debug($g->{matches}, " matches");
+#	$s->debug("0 .. ", (scalar(@out) - 1));
 
 	\@out;
 }
