@@ -1,14 +1,11 @@
 # Http.pm:  interface to cgi protocol
 #
-# $Id: Http.pm,v 1.13 1998/08/11 18:12:39 mike Exp $
+# $Id: Http.pm,v 1.6 1997/06/17 04:22:52 mike Exp $
 #
 package Vend::Http;
 
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
 #
-# Modified by and
-# Copyright 1997-1998, Mike Heins <mikeh@minivend.com>
-#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -27,13 +24,11 @@ package Vend::Http::Base;
 # abstract class
 
 use strict;
-use vars qw($VERSION);
-$VERSION = substr(q$Revision: 1.13 $, 10);
 
 #sub Server                    { $_[0]->{'Server'} }
-sub Content_Encoding          { $_[0]->{'Content_Encoding'} }
+#sub Content_Encoding          { $_[0]->{'Content_Encoding'} }
 #sub Content_Language          { $_[0]->{'Content_Language'} }
-sub Content_Transfer_Encoding { $_[0]->{'Content_Transfer_Encoding'} }
+#sub Content_Transfer_Encoding { $_[0]->{'Content_Transfer_Encoding'} }
 #sub Server_Software           { $_[0]->{'Server_Software'} }
 #sub URI                      { $_[0]->{'URI'} }
 sub Accept                    { $_[0]->{'Accept'} }
@@ -55,9 +50,7 @@ sub Derived_From              { $_[0]->{'Derived_From'} }
 sub Expires                   { $_[0]->{'Expires'} }
 sub Forwarded                 { $_[0]->{'Forwarded'} }
 sub From                      { $_[0]->{'From'} }
-#sub Glue	              	  { $_[0]->{'Glue'} }
 sub HTTP_Version              { $_[0]->{'HTTP_Version'} }
-sub Http_Host                 { $_[0]->{'Http_Host'} }
 sub Https_on              	  { $_[0]->{'Https_on'} }
 sub If_Modified_Since         { $_[0]->{'If_Modified_Since'} }
 sub Last_Modified             { $_[0]->{'Last_Modified'} }
@@ -125,7 +118,6 @@ my @Map =
     (
      'Authenticated_User' => 'REMOTE_USER',
      'Authorization_Type' => 'AUTH_TYPE',
-     'Authorization' => 'AUTHORIZATION',
      'Client_Hostname' => 'REMOTE_HOST',
      'Client_IP_Address' => 'REMOTE_ADDR',
      'Client_Ident' => 'REMOTE_IDENT',
@@ -133,13 +125,10 @@ my @Map =
      'Content_Type' => 'CONTENT_TYPE',
      'Cookie' => 'HTTP_COOKIE',
      'From' => 'HTTP_FROM',
-#     'Glue' => 'MV_GLUE',
      'Https_on' => 'HTTPS',
-     'Http_Host' => 'HTTP_HOST',
      'Method', => 'REQUEST_METHOD',
      'Path_Info' => 'PATH_INFO',
      'Path_Translated' => 'PATH_TRANSLATED',
-	 'Pragma' => 'HTTP_PRAGMA',
      'Query' => 'QUERY_STRING',
      'Reconfigure' => 'RECONFIGURE_MINIVEND',
      'Referer' => 'HTTP_REFERER',
@@ -147,15 +136,16 @@ my @Map =
      'Server_Host' => 'SERVER_NAME',
      'Server_Port' => 'SERVER_PORT',
      'User_Agent' => 'HTTP_USER_AGENT',
-     'Content_Encoding' => 'HTTP_CONTENT_ENCODING',
+#     'Content_Encoding' => 'HTTP_CONTENT_ENCODING',
 #     'Content_Language' => 'HTTP_CONTENT_LANGUAGE',
-     'Content_Transfer_Encoding' => 'HTTP_CONTENT_TRANSFER_ENCODING',
+#     'Content_Transfer_Encoding' => 'HTTP_CONTENT_TRANSFER_ENCODING',
 #     'Server_Software' => 'SERVER_SOFTWARE',
 # Accept
 # Accept_Charset
 # Accept_Encoding
 # Accept_Language
 # Allow
+# Authorization
 # Date
 # Derived_From
 # Expires
@@ -167,6 +157,7 @@ my @Map =
 # Location
 # MIME_Version
 # Message_ID
+# Pragma
 # Reason_Phrase
 # Referer
 # Request_URI
@@ -225,6 +216,7 @@ sub read_entity_body {
 package Vend::form;
 
 use strict;
+#use Vend::Uneval;
 
 sub unhexify {
     my($s) = @_;
