@@ -1,6 +1,6 @@
 package Vend::Parser;
 
-# $Id: Parser.pm,v 1.16 1999/06/07 08:07:30 mike Exp mike $
+# $Id: Parser.pm,v 1.5 1997/06/17 04:22:52 mike Exp $
 
 =head1 NAME
 
@@ -50,6 +50,7 @@ third argument is a reference to an array with the lower case
 attribute keys in the original order.  The fourth argument is the
 original MiniVend page.
 
+
 =item $self->end($tag)
 
 This method is called when an end tag has been recognized.  The
@@ -90,9 +91,9 @@ Copyright 1996 Gisle Aas. All rights reserved.
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-Modified for use by MiniVend.
+Modified for use by MiniVend
 
-Copyright 1997-1998 Mike Heins.  
+Copyright 1997 Mike Heins.  
 
 =head1 AUTHOR
 
@@ -106,7 +107,7 @@ use strict;
 
 use HTML::Entities ();
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
 
 
 sub new
@@ -157,9 +158,22 @@ sub parse
 	# it is a token or not, or the $$buf is empty.
 	while (1) {  # the loop will end by returning when text is parsed
 		# First we try to pull off any plain text (anything before a "<" char)
-		if ($$buf =~ s|^([^\[<]+)||) {
-			$self->text($1);
-			return $self unless length $$buf;
+		if ($$buf =~ s|^([^\[]+)||) {
+			unless (length $$buf) {
+				my $text = $1;
+				# At the end of the buffer, we should not parse white space
+				# but leave it for parsing on the next round.
+				if ($text =~ s|(\s+)$||) {
+					$$buf = $1;
+				# Same treatment for chopped up entites.
+				} elsif ($text =~ s/(&(?:(?:\#\d*)?|\w*))$//) {
+					$$buf = $1;
+				};
+				$self->text($text);
+				return $self;
+			} else {
+				$self->text($1);
+			}
 		# Netscapes buggy comments are easy to handle
 		} elsif ($self->{'_netscape_comment'} && $$buf =~ m|^(<!--)|) {
 			if ($$buf =~ s|^<!--(.*?)-->||s) {
@@ -168,6 +182,34 @@ sub parse
 				return $self;  # must wait until we see the end of it
 			}
 		# Then, markup declarations (usually either <!DOCTYPE...> or a comment)
+#		} elsif ($$buf =~ s|^(\[!)||) {
+#			my $eaten = $1;
+#			my $text = '';
+#			my @com = ();  # keeps comments until we have seen the end
+#			# Eat text and beginning of comment
+#			while ($$buf =~ s|^(([^>]*?)--)||) {
+#				$eaten .= $1;
+#				$text .= $2;
+#				# Look for end of comment
+#				if ($$buf =~ s|^((.*?)--)||s) {
+#					$eaten .= $1;
+#					push(@com, $2);
+#				} else {
+#					# Need more data to get all comment text.
+#					$$buf = $eaten . $$buf;
+#					return $self;
+#				}
+#			}
+#			# Can we finish the tag
+#			if ($$buf =~ s|^([^>]*)>||) {
+#				$text .= $1;
+#				$self->declaration($text) if $text =~ /\S/;
+#				# then tell about all the comments we found
+#				for (@com) { $self->comment($_); }
+#			} else {
+#				$$buf = $eaten . $$buf;  # must start with it all next time
+#				return $self;
+#			}
 		} elsif ($$buf =~ s|^\[/||) {
 			# end tag
 			if ($$buf =~ s|^\s*([a-z][-a-z0-9._]*)\s*\]||i) {
@@ -181,19 +223,9 @@ sub parse
 				$$buf = "";
 			}
 		# Then, finally we look for a start tag
-		} elsif ($$buf =~ s|^([<\[])||) {
+		} elsif ($$buf =~ s|^\[||) {
 			# start tag
-			my $eaten = $1;
-			my $end_brack;
-			if($eaten eq '[') {
-				$self->{HTML} = 0;
-				$end_brack = '[^\]]'
-			}
-			else {
-				$self->{HTML} = 1;
-				$end_brack = '[^>]'
-			}
-
+			my $eaten = '[';
 
 			# This first thing we must find is a tag name.  RFC1866 says:
 			#   A name consists of a letter followed by letters,
@@ -202,104 +234,56 @@ sub parse
 			#   the SGML declaration for HTML, 9.5, "SGML Declaration
 			#   for HTML".  In a start-tag, the element name must
 			#   immediately follow the tag open delimiter `<'.
-			if ($$buf =~ s|^(([a-zA-Z][-a-zA-Z0-9._]*)((?:\s+$end_brack+)?\s+[Mm][Vv]\s*=)?\s*)||) {
+			if ($$buf =~ s|^(([a-zA-Z][-a-zA-Z0-9._]*)\s*)||) {
 				$eaten .= $1;
-
-				my ($tag, $end_tag);
+				my $tag = lc $2;
 				my %attr;
 				my @attrseq;
-				my $old;
-
-				if(! $3) {
-					$tag = lc $2;
-					($self->text($eaten), next)
-						if $self->{HTML};
-				}
-				else {
-					$end_tag = $2;
-					( $$buf =~ s|^((['"])(.*?)\2\s*)||s and $tag = $3 )
-					or
-					( $$buf =~ s|^(([a-zA-Z][-a-zA-Z0-9._]*)\s*)|| and $tag = $2)
-					or ($self->text($eaten), next);
-					$eaten .= $1;
-					if( index($tag, " ") != -1 ) {
-						($tag, $attr{OLD}) = split /\s+/, $tag, 2;
-						$old = 1;
-					}
-					$tag = lc $tag;
-				}
-				my $nopush;
-				my $urldecode = ($tag =~ s/^urld?(?:ecode)?$/urldecode/) ? 1 : 0;
 
 				# Then we would like to find some attributes
 				#
 				# Arrgh!! Since stupid Netscape violates RCF1866 by
 				# using "_" in attribute names (like "ADD_DATE") of
 				# their bookmarks.html, we allow this too.
-				while (	$$buf =~ s|^(([a-zA-Z][-a-zA-Z0-9._]*)\s*)|| or
-					 	! $self->{HTML} && $$buf =~ s|^(([=!<>][=~]?)\s+)||) {
+				my $old = 0;
+				while ($$buf =~ s|^(([a-zA-Z][-a-zA-Z0-9._]*)\s*)||) {
 					$eaten .= $1;
 					my $attr = lc $2;
-					$attr =~ s/^[Mm][Vv]\.?// || undef $attr
-						if $self->{HTML} and ! $urldecode;
-						
 					my $val;
-					
+					$old = 0;
 					# The attribute might take an optional value (first we
 					# check for an unquoted value)
-					if ($$buf =~ s|(^=\s*([^\"\'\]\s][^\]>\s]*)\s*)||) {
+					if ($$buf =~ s|(^=\s*([^\"\'\]\s][^\]\s]*)\s*)||) {
 						$eaten .= $1;
-						next unless defined $attr;
 						$val = $2;
 						HTML::Entities::decode($val);
 					# or quoted by " or '
 					} elsif ($$buf =~ s|(^=\s*([\"\'])(.*?)\2\s*)||s) {
 						$eaten .= $1;
-						next unless defined $attr;
 						$val = $3;
 						HTML::Entities::decode($val);
 					# truncated just after the '=' or inside the attribute
-					} elsif ($$buf =~ m|^(=\s*)$|s or
+					} elsif ($$buf =~ m|^(=\s*)$| or
 							 $$buf =~ m|^(=\s*[\"\'].*)|s) {
 						$$buf = "$eaten$1";
 						return $self;
-					} elsif (!$old) {
-						# assume attribute with implicit value, but
-						# if not,no value is set and the
-						# eaten value is grown
-						undef $nopush;
-						($attr,$val,$nopush) = $self->implicit($tag,$attr);
-						$old = 1 unless $val || $self->{HTML};
-
 					} else {
-# DEBUG
-#Vend::Util::logDebug
-#("Abort attribute parsing, attr='$attr'.\n")
-#	if ::debug(0x2);
-# END DEBUG
+						# assume attribute with implicit value, which 
+						# means in MiniVend no value is set and the
+						# eaten value is grown
 						$old = 1;
 					}
-					next if $old and !$self->{HTML};
-					if(! $attr) {
-						$attr->{OLD} = $val if defined $attr;
-						next;
-					}
+					next if $old;
 					$attr{$attr} = $val;
-					push(@attrseq, $attr) unless $nopush;
+					push(@attrseq, $attr);
 				}
 
-				# At the end there should be a closing "\] or >"
-				if ($$buf =~ s|^\]|| ) {
-					$self->text("$eaten]"), next
-						if $self->{HTML};
+				# At the end there should be a closing "\]"
+				if ($$buf =~ s|^\]||) {
 					$self->start($tag, \%attr, \@attrseq, "$eaten]");
-				} elsif ($$buf =~ s|^>|| ) {
-					$self->text("$eaten>"), next
-						unless $self->{HTML};
-					$self->start($tag, \%attr, \@attrseq, "$eaten>", $end_tag);
 				} elsif ($$buf =~ s|^([^\]\n]+\])||) {
 					$eaten .= $1;
-					$self->start($tag, {}, [], $eaten, $end_tag);
+					$self->start($tag, {}, [], $eaten);
 				} elsif (length $$buf) {
 					# Not a conforming start tag, regard it as normal text
 					$self->text($eaten);
@@ -321,16 +305,61 @@ sub parse
 			# The buffer is empty now
 			return $self;
 		}
-		return $self if $self->{SEND};
 	}
 	$self;
 }
 
+sub netscape_buggy_comment
+{
+	my $self = shift;
+	my $old = $self->{'_netscape_comment'};
+	$self->{'_netscape_comment'} = shift if @_;
+	return $old;
+}
+
+sub parse_file
+{
+	my($self, $file) = @_;
+	no strict 'refs';  # so that a symbol ref as $file works
+	local(*F);
+	unless (ref($file) || $file =~ /^\*[\w:]+$/) {
+		# Assume $file is a filename
+		open(F, $file) || die "Can't open $file: $!";
+		$file = \*F;
+	}
+	my $chunk = '';
+	while(read($file, $chunk, 2048)) {
+		$self->parse($chunk);
+	}
+	close($file);
+	$self->eof;
+}
+
+sub text
+{
+	# my($self, $text) = @_;
+}
+
+sub declaration
+{
+	# my($self, $decl) = @_;
+}
 
 sub comment
 {
 	# my($self, $comment) = @_;
 }
 
+sub start
+{
+	my($self, $tag, $attr, $attrseq, $origtext) = @_;
+	# $attr is reference to a HASH, $attrseq is reference to an ARRAY
+}
+
+sub end
+{
+	my($self, $tag) = @_;
+}
+
 1;
 __END__
