@@ -1,32 +1,17 @@
-# Session.pm - Minivend Sessions
-#
-# $Id: Session.pm,v 1.36 1999/06/07 08:07:49 mike Exp mike $
-# 
-# Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
-# Copyright 1996-1999 by Michael J. Heins <mikeh@iac.net>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-#
+# $Id: Session.pm,v 1.13 1997/08/26 20:45:39 mike Exp mike $
 
 package Vend::Session;
 require Exporter;
 
-use vars qw($VERSION);
-$VERSION = substr(q$Revision: 1.36 $, 10);
+# AUTOLOAD
+#use AutoLoader;
+#@ISA = qw(Exporter AutoLoader);
+#*AUTOLOAD = \&AutoLoader::AUTOLOAD;
+# END AUTOLOAD
 
+# NOAUTO
 @ISA = qw(Exporter);
+# END NOAUTO
 
 @EXPORT = qw(
 
@@ -43,10 +28,10 @@ session_name
 );
 
 use strict;
+use Carp;
 use Fcntl;
 use Vend::Util;
 
-require Vend::SessionDB;
 
 BEGIN {
 	if($Global::GDBM) {
@@ -62,21 +47,23 @@ BEGIN {
 	}
 }
 
-my ($Session_open, $File_sessions, $DB_sessions, $DB_object);
+# AUTOLOAD
+#use vars qw($Session_open $File_sessions);
+# END AUTOLOAD
+
+# NOAUTO
+my ($Session_open, $File_sessions);
+# END NOAUTO
 
 
 # Selects based on initial config
 if($Global::GDBM) {
 	$File_sessions = 0;
 	$Session_open = sub {
-		$DB_object = tie(%Vend::SessionDBM,
-			'GDBM_File',
+		 tie(%Vend::SessionDBM, 'GDBM_File',
 			$Vend::Cfg->{'SessionDatabase'} . ".gdbm",
-			&GDBM_WRCREAT,
-			$Vend::Cfg->{'FileCreationMask'}
-			);
-		die "Could not tie to $Vend::Cfg->{'SessionDatabase'}: $!\n"
-			unless defined $DB_object;
+			&GDBM_WRCREAT, $Vend::Cfg->{'FileCreationMask'})
+		or die "Could not tie to $Vend::Cfg->{'SessionDatabase'}: $!\n";
 	};
 }
 elsif($Global::DB_File) {
@@ -97,7 +84,7 @@ else {
 }
 
 
-# SESSIONS implemented using DBM
+## SESSIONS implemented using DBM
 
 sub get_session {
 	$Vend::HaveSession = 0;
@@ -135,16 +122,6 @@ sub put_session {
 
 sub open_session {
 	return 1 if defined $Vend::SessionOpen;
-	if($Vend::Cfg->{SessionDB}) {
-		$Session_open = sub {
-			return 1 if $DB_sessions;
-			tie(%Vend::SessionDBM,
-				'Vend::SessionDB',
-				$Vend::Cfg->{SessionDB})
-			or die "Could not tie to $Vend::Cfg->{SessionDB}: $!\n";
-			$DB_sessions = 1;
-		};
-	}
 	unless($File_sessions) {
 		open(Vend::SessionLock, "+>>$Vend::Cfg->{'SessionLockFile'}")
 			or die "Could not open '$Vend::Cfg->{'SessionLockFile'}': $!\n";
@@ -162,15 +139,10 @@ sub new_session {
     my($seed) = @_;
     my($name);
 
-# DEBUG
-#Vend::Util::logDebug
-#("new session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("new session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
 	open_session();
     for (;;) {
 		$Vend::SessionID = random_string() unless defined $seed;
-		next if $Vend::RetireDBM{$Vend::SessionID};
 		undef $seed;
 		$name = session_name();
 		unless ($File_sessions) { 
@@ -180,37 +152,66 @@ sub new_session {
 			last unless exists $Vend::SessionDBM{$name};
 		}
     }
-	$CGI::cookie = $Vend::Cookie = '';
+	$CGI::cookie = '';
     $Vend::SessionName = $name;
+	#add_session_marker();
     init_session();
 	return if $File_sessions;
 	write_session();
 	close_session();
 }
 
-sub check_override {
-	return 1 unless defined $CGI::values{mv_override_domain};
-	return 1 if $CGI::values{mv_override_check} eq $Vend::Session->{override_check};
-	logError "Override check '" .
-		$Vend::Session->{override_check} . "' not good from $CGI::original_host.\n";
-	logGlobal "Override check '" .
-		$Vend::Session->{override_check} . "' not good from $CGI::original_host.\n";
-	die "Security violation. Check error log $Global::ErrorFile\n";
+sub save_session {
+	my($source,$dest,$name) = @_;
+	my($s,$d,$found);
+#print("save session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
+	$dest = $dest || $Vend::SessionID;
+	$name = $name || time;
+	open_session();
+	unless ($File_sessions) { 
+		$found = defined $Vend::SessionDBM{$source};
+	}
+	else {
+		$found = exists $Vend::SessionDBM{$source};
+	}
+
+	unless($found) {
+		close_session();
+		return '';
+	}
+
+	$s = eval $Vend::SessionDBM{$source};
+    die "Could not eval '$s' from session dbm: $@\n" if $@;
+	
+	unless ($File_sessions) { 
+		$found = defined $Vend::SessionDBM{$dest};
+	}
+	else {
+		$found = exists $Vend::SessionDBM{$dest};
+	}
+
+	if ($found) {
+		$d = eval $Vend::SessionDBM{$dest};
+		die "Could not eval '$d' from session dbm: $@\n" if $@;
+	}
+	else {
+		$d = {};
+	}
+
+	$d->{$name} = $s;
+
+	$Vend::SessionDBM{$dest} = uneval($d);
+
+	close_session();
 }
 
-sub close_session {
-# DEBUG
-#Vend::Util::logDebug
-#("close session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
-	return 1 if ! defined $Vend::SessionOpen;
 
-	unless($DB_sessions) {
-		undef $DB_object;
-		untie %Vend::SessionDBM
-			or die "Could not close $Vend::Cfg->{'SessionDatabase'}: $!\n";
-	}
+
+sub close_session {
+#print("close session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
+	return 1 if ! defined $Vend::SessionOpen;
+    untie %Vend::SessionDBM
+		or die "Could not close $Vend::Cfg->{'SessionDatabase'}: $!\n";
 	
 	return 1 if $File_sessions;
 
@@ -223,41 +224,28 @@ sub close_session {
 
 sub write_session {
     my($s);
-# DEBUG
-#Vend::Util::logDebug
-#("write session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("write session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
 	my $time = time;
     $Vend::Session->{'time'} = $time;
-    my $save = $Vend::Session->{'user'};
     undef $Vend::Session->{'user'};
-    #undef $Vend::Session->{'arg'};
+    undef $Vend::Session->{'arg'};
 	undef $Vend::Session->{'items'};
 	undef $Vend::Session->{'values'}->{mv_change_frame};
-    $s = uneval_fast($Vend::Session);
+    $s = uneval($Vend::Session);
     $Vend::SessionDBM{$Vend::SessionName} = $s or 
 		die "Data was not stored in DBM file\n";
-    $Vend::Session->{'user'} = $save;
+		#if $Vend::SessionDBM{$Vend::SessionName} ne $s;
 }
 
 sub unlock_session {
-# DEBUG
-#Vend::Util::logDebug
-#("unlock session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("unlock session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
 	delete $Vend::SessionDBM{'LOCK_' . $Vend::SessionName}
 		unless $File_sessions;
 }
 
 sub lock_session {
 	return 1 if $File_sessions;
-# DEBUG
-#Vend::Util::logDebug
-#("lock session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("lock session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
 	my $lockname = 'LOCK_' . $Vend::SessionName;
 	my ($tried, $locktime, $sleepleft, $pid, $now, $left);
 	$tried = 0;
@@ -271,8 +259,7 @@ sub lock_session {
 			$left = $now - $locktime;
 			if ( $left > $Global::HammerLock ) {
 				$Vend::SessionDBM{$lockname} = "$now:$$";
-#				logError("Hammered session lock $lockname left by PID $pid");
-				logError( errmsg('Session.pm:1', "Hammered session lock %s left by PID %s" , $lockname, $pid) );
+				logError("Hammered session lock $lockname left by PID $pid");
 				return 1;
 			}
 			elsif ($left < 0) {
@@ -314,32 +301,12 @@ EOF
 sub read_session {
     my($s);
 
-# DEBUG
-#Vend::Util::logDebug
-#("read session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("read session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
 	$s = $Vend::SessionDBM{$Vend::SessionName};
-# DEBUG
-#Vend::Util::logDebug
-#("Session:\n$s\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("Session:\n$s\n") if $Global::DEBUG;
 	return new_session($Vend::SessionID) unless $s;
-    $Vend::Session = evalr($s);
+    $Vend::Session = eval($s);
     die "Could not eval '$s' from session dbm: $@\n" if $@;
-
-	$Vend::Session->{host} = $CGI::host;
-
-	# The below can die if there is an override and the hosts/checks
-	# don't match
-	check_override();
-
-	$Vend::Session->{'arg'} = $Vend::Argument;
-	$Vend::Session->{override_check} = $Vend::Session->{'time'};
-
-    $::Values	= $Vend::Session->{'values'};
-    $::Scratch	= $Vend::Session->{'scratch'};
     $Vend::Items	= $Vend::Session->{'items'}
 					= $Vend::Session->{'carts'}->{'main'};
 }
@@ -347,20 +314,12 @@ sub read_session {
 
 ## SESSIONS
 
-my $joiner = $Global::Windows ? '_' : ':';
-
 sub session_name {
     my($host, $user, $fn, $proxy);
 
 	if(defined $CGI::user and $CGI::user) {
 		$host = escape_chars($CGI::user);
 	}
-	elsif(defined $CGI::cookieuser) {
-		$host = $CGI::cookieuser;
-	}
-	elsif(defined $CGI::cookiehost) {
-		$host = $CGI::cookiehost;
-	}
 	else {
 		$host = $CGI::host;
 		$proxy = index($host,"proxy");
@@ -368,43 +327,63 @@ sub session_name {
 			if ($proxy >= 0);
 		$host = escape_chars($host);
 	}
-# DEBUG
-#Vend::Util::logDebug
-#("name session user=$CGI::user host=$host ($CGI::host)\n")
-#	if ::debug(0x20);
-# END DEBUG
-    $fn = $Vend::SessionID . $joiner . $host;
-# DEBUG
-#Vend::Util::logDebug
-#("name session id=$Vend::SessionID  name=$fn\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("name session user=$CGI::user host=$host ($CGI::host)\n") if $Global::DEBUG;
+    $fn = $Vend::SessionID . ':' . $host;
+#print("name session id=$Vend::SessionID  name=$fn\n") if $Global::DEBUG;
     $fn;
 }
 
 
 sub init_session {
-# DEBUG
-#Vend::Util::logDebug
-#("init session id=$Vend::SessionID  name=$Vend::SessionName\n")
-#	if ::debug(0x20);
-# END DEBUG
+#print("init session id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
     $Vend::Session = {
 	'version' => 1,
-	'frames' => $Vend::Cfg->{FramesDefault},
-	'arg' => $Vend::Argument,
+	'frames' => 0,
+	'login' => '',
 	'browser' => $CGI::useragent,
 	'referer' => $CGI::referer,
-	'scratch' => { 'mv_no_cache' => 1 },
+	'scratch' => {},
 	'values' => {},
 	'carts' => {main => []},
     };
     $Vend::Items	= $Vend::Session->{'items'}
 					= $Vend::Session->{'carts'}->{'main'};
-	$::Values = $Vend::Session->{'values'};
-	$::Scratch = $Vend::Session->{'scratch'};
 	$Vend::Session->{'secure'} = $Vend::Cfg->{'AlwaysSecure'} ? 1 : 0;
-	$::Values->{'mv_shipmode'} = $Vend::Cfg->{'DefaultShipping'};
+	$Vend::Session->{'values'}->{'mv_shipmode'} = $Vend::Cfg->{'DefaultShipping'};
+}
+
+# AUTOLOAD
+#1;
+#__END__
+# END AUTOLOAD
+
+sub remove_session_marker {
+	my ($id) = @_;
+	my ($name);
+	if(defined $id) {
+		$id =~ /^(\w{8}):(.*)/;
+		$id = $1; $name = $2;
+	}
+	else {
+		$id = $Vend::SessionID;
+		$name = $Vend::SessionName;
+	}
+
+	return undef unless $name and $id;
+
+	my $s = eval $Vend::SessionDBM{$id};
+    die "Could not eval '$s' from session dbm: $@\n" if $@;
+	delete $s->{$name};
+	$Vend::SessionDBM{$id} = uneval $s;
+}
+
+sub add_session_marker {
+	my $s = eval $Vend::SessionDBM{$Vend::SessionID};
+    die "Could not eval '$s' from session dbm: $@\n" if $@;
+#print("Add session marker id=$Vend::SessionID  name=$Vend::SessionName\n") if $Global::DEBUG;
+	$s = {} unless $s;
+	$s->{$Vend::SessionName} = {};
+	$Vend::SessionDBM{$Vend::SessionID} = uneval $s;
 }
 
 sub dump_sessions {
@@ -414,17 +393,17 @@ sub dump_sessions {
 	my $pretty;
 
 	eval {	require Data::Dumper;
+			import Data::Dumper 'DumperX';
 			$Data::Dumper::Indent = 3;
 			$Data::Dumper::Terse = 1; };
 	$pretty = $@ ? 0 : 1;
     open_session();
     while(($session_name, $s) = each %Vend::SessionDBM) {
-		next if $session_name eq 'dumpprog:DUMP';
 		if(defined $called) {
 			next unless $session_name =~ /$called/;
 		}
-		if ($pretty or defined $Storable::VERSION) {
-			my $ref = evalr $s;
+		if ($pretty) {
+			my $ref = eval $s;
 			$s = uneval($ref);
 		}
 		print "$session_name $s\n\n";
@@ -432,14 +411,7 @@ sub dump_sessions {
     close_session();
 }
 
-sub reorg {
-	return unless $DB_object;
-	GDBM_File::reorganize($DB_object);
-	GDBM_File::sync($DB_object);
-}
-
 sub expire_sessions {
-	my ($reorg) = @_;
     my($time, $session_name, $s, $session, @delete);
 
     $time = time;
@@ -455,18 +427,19 @@ sub expire_sessions {
 
 		# Session markers
 		if ($session_name =~ /^\w{8}$/) {
-			$session = evalr ($s);
+			$session = eval ($s);
 			die "Could not eval '$s' from session dbm: $@\n" if $@;
 			next if keys %$session;   # Don't remove if has session marker
 			push @delete, $session_name;
 		}
 
-		$session = evalr($s);
+		$session = eval($s);
 		die "Could not eval '$s' from session dbm: $@\n" if $@;
-		next if check_save($time);
+		next if check_save($session,$time);
 		if ( (! defined $session) ||
 			 $time - $session->{'time'} > $Vend::Cfg->{'SessionExpire'}) {
 			push @delete, $session_name;
+			remove_session_marker($session_name);
 		}
     }
     foreach $session_name (@delete) {
@@ -483,12 +456,12 @@ sub expire_sessions {
 		}
 		closedir(Vend::DELDIR);
     }
-	reorg() if $reorg;
     close_session();
 }
 
 sub check_save {
 	my($time) = (@_);
+	return 0 unless $Vend::Session->{'values'}->{mv_save_session};
 	my $expire;
 
 	$time = $time || time();
@@ -497,13 +470,18 @@ sub check_save {
 		$expire = $Vend::Session->{'values'}->{mv_expire_time};
 		unless($expire =~ /^\d{6,}$/) {
 			$expire = Vend::Config::time_to_seconds($expire);
+			if(defined $expire) {
+				$Vend::Session->{'values'}->{mv_expire_time} = $expire;
+			}
 		}
 	}
-	$expire = $Vend::Cfg->{SaveExpire} unless $expire;
+	else {
+		$expire = $Vend::Cfg->{SaveExpire};
+	}
 
-	$Vend::Session->{'expire'} = $Vend::Expire = $time + $expire;
+	$Vend::Expire = $time + $expire;
 
-	return ($expire > $time);
+	return ($time - $expire < $Vend::Session->{'time'}) ? 1 : 0;
 }	
 
 1;
