@@ -1,9 +1,9 @@
 # SessionFile.pm:  stores session information in files
 #
-# $Id: SessionFile.pm,v 1.7 1998/01/31 05:22:20 mike Exp $
+# $Id: SessionFile.pm,v 1.3 1997/08/26 20:45:39 mike Exp mike $
 #
 # Copyright 1996 by Andrew M. Wilcox <awilcox@world.std.com>
-# Copyright 1996-1998 by Michael J. Heins <mikeh@iac.net>
+# Copyright 1996 by Michael J. Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 
-# $Id: SessionFile.pm,v 1.7 1998/01/31 05:22:20 mike Exp $
+# $Id: SessionFile.pm,v 1.3 1997/08/26 20:45:39 mike Exp mike $
 
 package Vend::SessionFile;
 require Tie::Hash;
@@ -29,9 +29,6 @@ require Tie::Hash;
 use strict;
 use Vend::Util;
 
-use vars qw($VERSION);
-$VERSION = substr(q$Revision: 1.7 $, 10);
-
 my $SessionDir;
 my $SessionFile;
 my $SessionLock;
@@ -52,19 +49,13 @@ sub FETCH {
     $SessionLock = $SessionDir . "/LOCK_$key";
 	return undef unless -f $SessionFile;
 	my $str;
-	if($Global::Windows) {
-		open SESSION, "< $SessionFile"
-			or return '';
-	}
-	else {
-		open(SESSIONLOCK, "+>>$SessionLock")
-			or die "Can't open '$SessionLock': $!\n";
-		open(SESSION, "+>>$SessionFile")
-			or die "Can't open '$SessionFile': $!\n";
-		lockfile(\*SESSION, 1, 1);
+    open(SESSIONLOCK, "+>>$SessionLock")
+        or die "Can't open '$SessionLock': $!\n";
+    open(SESSION, "+>>$SessionFile")
+        or die "Can't open '$SessionFile': $!\n";
+    lockfile(\*SESSION, 1, 1);
 
-		seek(SESSION, 0, 0) or die "Can't seek session: $!\n";
-	}
+    seek(SESSION, 0, 0) or die "Can't seek session: $!\n";
     while(<SESSION>) {
 		$str .= $_;
 	}
@@ -98,7 +89,6 @@ sub DELETE {
     my $lockname = $SessionDir . "/LOCK_$key";
 	(warn ("SessionFile.pm: $key not found.\n"), return 0) unless -f $filename;
 	unlink $filename or die "Couldn't delete key $key: $!\n";
-	return 1 if $Global::Windows;
 	unlink $lockname or die "Couldn't delete lock for key $key: $!\n";
 }
 
@@ -108,14 +98,8 @@ sub STORE {
     $SessionLock = $SessionDir . "/LOCK_$key";
 	close(SESSION);
     unlink $SessionFile;
-	if($Global::Windows) {
-		open SESSION, ">$SessionFile"
-        or die "Can't write '$SessionFile': $!\n";
-	}
-	else {
-		open(SESSION, "+>$SessionFile")
-			or die "Can't open '$SessionFile': $!\n";
-	}
+    open(SESSION, "+>$SessionFile")
+        or die "Can't open '$SessionFile': $!\n";
     #seek(SESSION, 0, 0) or die "Can't seek session: $!\n";
     #lockfile(\*SESSION, 1, 1);
 	print SESSION $val;
