@@ -1,10 +1,10 @@
 # Table/DB_File.pm: access a table stored in a DB file hash
 #
-# $Id: DB_File.pm,v 1.17 1999/06/07 08:08:20 mike Exp mike $
+# $Id: DB_File.pm,v 1.7 1997/05/22 07:10:45 mike Exp $
 #
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
 #
-# Modified 1996-1998 by Mike Heins <mikeh@iac.net>
+# Modified 1996 by Mike Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,8 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 package Vend::Table::DB_File;
-$VERSION = substr(q$Revision: 1.17 $, 10);
+$VERSION = substr(q$Revision: 1.7 $, 10);
+use Carp;
 use strict;
 use Fcntl;
 use DB_File;
@@ -34,20 +35,9 @@ my @Hex_string;
     }
 }
 
-use vars qw($Storable);
-
-# See if we can do Storable
-BEGIN {
-	eval {
-		die unless $ENV{MINIVEND_STORABLE_DB};
-		require Storable;
-		$Storable = 1;
-	};
-}
-
-
 sub stuff {
     my ($val) = @_;
+
     $val =~ s,([\t\%]),$Hex_string[ord($1)],eg;
     return $val;
 }
@@ -64,29 +54,30 @@ sub unstuff {
 # 2: column index
 # 3: tie hash
 # 4: dbm object
-# 5: configuration
 
-my ($FILENAME, $COLUMN_NAMES, $COLUMN_INDEX, $TIE_HASH, $DBM, $CONFIG) = (0 .. 5);
+my ($FILENAME, $COLUMN_NAMES, $COLUMN_INDEX, $TIE_HASH, $DBM) = (0 .. 4);
 
-sub config {
-	my ($self, $key, $value) = @_;
-	return $self->[$CONFIG]{$key} unless defined $value;
-	$self->[$CONFIG]{$key} = $value;
+sub create_table {
+    my ($class, $config, $filename, $columns) = @_;
+
+    return $class->create($columns, $filename, $config);
 }
 
 sub create {
-    my ($class, $config, $columns, $filename) = @_;
+    my ($class, $columns, $filename, $config) = @_;
 
     $config = {} unless defined $config;
-    my $File_permission_mode = $config->{File_permission_mode} || 0666;
+    my ($File_permission_mode)
+        = $$config{'File_permission_mode'};
+    $File_permission_mode = 0666 unless defined $File_permission_mode;
 
-    die "columns argument $columns is not an array ref\n"
+    croak "columns argument $columns is not an array ref\n"
         unless ref($columns) eq 'ARRAY';
 
     # my $column_file = "$filename.columns";
     # my @columns = @$columns;
     # open(COLUMNS, ">$column_file")
-    #    or die "Couldn't create '$column_file': $!";
+    #    or croak "Couldn't create '$column_file': $!";
     # print COLUMNS join("\t", @columns), "\n";
     # close(COLUMNS);
 
@@ -97,21 +88,20 @@ sub create {
     }
 
     my $tie = {};
-    my $flags = O_RDWR | O_CREAT;
-
+    my $flags = O_RDWR|O_CREAT;
     my $dbm = tie(%$tie, 'DB_File', $filename, $flags, $File_permission_mode)
-        or die "Could not create '$filename': $!";
+        or croak "Could not create '$filename': $!";
 
     $tie->{'c'} = join("\t", @$columns);
 
-    my $self = [$filename, $columns, $column_index, $tie, $dbm, $config];
+    my $self = [$filename, $columns, $column_index, $tie, $dbm];
     bless $self, $class;
 }
 
 
 sub open_table {
     my ($class, $config, $filename) = @_;
-    my ($Read_only) = $config->{Read_only};
+    my ($Read_only) = $$config{'Read_only'};
 
     my $tie = {};
 
@@ -122,14 +112,10 @@ sub open_table {
     else {
         $flags = O_RDWR;
     }
-# DEBUG
-#Vend::Util::logDebug
-#("DB_File flags are: '$flags'\n")
-#	if ::debug(0x4);
-# END DEBUG
+	print("Flags are: '$flags'\n") if $Global::DEBUG;
 
     my $dbm = tie(%$tie, 'DB_File', $filename, $flags, 0600)
-        or die "Could not open '$filename': $!";
+        or croak "Could not open '$filename': $!";
 
     my $columns = [split(/\t/, $tie->{'c'})];
 
@@ -139,15 +125,14 @@ sub open_table {
         $column_index->{$columns->[$i]} = $i;
     }
 
-    my $self = [$filename, $columns, $column_index, $tie, $dbm, $config];
+    my $self = [$filename, $columns, $column_index, $tie, $dbm];
     bless $self, $class;
 }
 
 sub close_table {
     my ($s) = @_;
-	splice(@$s, $DBM, 1);
-	my $ref = splice(@$s, $TIE_HASH, 1);
-    untie %$ref or die "Could not close DB_File table $s->[$FILENAME]: $!\n";
+
+    untie %{$s->[$TIE_HASH]} or die "Could not close '$s->[$FILENAME]': $!\n";
 }
 
 
@@ -164,32 +149,17 @@ sub test_column {
 sub column_index {
     my ($s, $column) = @_;
     my $i = $s->[$COLUMN_INDEX]{$column};
-    die "There is no column named '$column'" unless defined $i;
+    croak "There is no column named '$column'" unless defined $i;
     return $i;
 }
 
-sub row_hash {
-    my ($s, $key) = @_;
-	my %row;
-	@row{ @{$s->[$COLUMN_NAMES]} } = $s->row($key);
-	return \%row;
-}
-
-sub unstuff_row {
+sub row {
     my ($s, $key) = @_;
     my $line = $s->[$TIE_HASH]{"k$key"};
-    die "There is no row with index '$key'" unless defined $line;
+    croak "There is no row with index '$key'" unless defined $line;
     return map(unstuff($_), split(/\t/, $line, 9999));
 }
 
-sub thaw_row {
-    my ($s, $key) = @_;
-    my $line = $s->[$TIE_HASH]{"k$key"};
-    die "There is no row with index '$key'" unless defined $line;
-    return @{ Storable::thaw($line) };
-}
-
-
 sub field_accessor {
     my ($self, $column) = @_;
     my $index = $self->column_index($column);
@@ -210,27 +180,12 @@ sub field_settor {
     };
 }
 
-sub stuff_row {
+sub set_row {
     my ($s, $key, @fields) = @_;
     my $line = join("\t", map(stuff($_), @fields));
     $s->[$TIE_HASH]{"k$key"} = $line;
 }
 
-sub freeze_row {
-    my ($s, $key, @fields) = @_;
-    $s->[$TIE_HASH]{"k$key"} = Storable::freeze(\@fields);
-}
-
-if($Storable) {
-	*set_row = \&freeze_row;
-	*row = \&thaw_row;
-}
-else {
-	*set_row = \&stuff_row;
-	*row = \&unstuff_row;
-}
-
-
 sub field {
     my ($s, $key, $column) = @_;
     return ($s->row($key))[$s->column_index($column)];
@@ -261,13 +216,13 @@ sub touch {
 
 sub each_record {
     my ($s) = @_;
-    my $key;
+    my ($key, $value);
 
     for (;;) {
-        $key = each %{$s->[3]};
+        ($key, $value) = each %{$s->[3]};
         if (defined $key) {
             if ($key =~ s/^k//) {
-                return ($key, $s->row($key));
+                return ($key, map(unstuff($_), split(/\t/, $value, 9999)));
             }
         }
         else {
@@ -289,8 +244,6 @@ sub record_exists {
     return $r;
 }
 
-*test_record = \&record_exists;
-
 sub delete_record {
     my ($s, $key) = @_;
 
