@@ -1,10 +1,9 @@
 # Table/GDBM.pm: access a table stored in a GDBM file
 #
-# $Id: GDBM.pm,v 1.11 1999/02/15 08:51:50 mike Exp mike $
+# $Id: GDBM.pm,v 1.3 1997/05/02 05:50:23 mike Exp $
 #
 
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
-# Copyright 1996-1999 by Mike Heins <mikeh@minivend.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,21 +20,11 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 package Vend::Table::GDBM;
-$VERSION = substr(q$Revision: 1.11 $, 10);
+$VERSION = substr(q$Revision: 1.3 $, 10);
+use Carp;
 use strict;
 use GDBM_File;
 
-use vars qw($Storable);
-
-# See if we can do Storable
-BEGIN {
-	eval {
-		die unless $ENV{MINIVEND_STORABLE_DB};
-		require Storable;
-		$Storable = 1;
-	};
-}
-
 my @Hex_string;
 {
     my $i;
@@ -46,6 +35,7 @@ my @Hex_string;
 
 sub stuff {
     my ($val) = @_;
+
     $val =~ s,([\t\%]),$Hex_string[ord($1)],eg;
     return $val;
 }
@@ -62,18 +52,17 @@ sub unstuff {
 # 2: column index
 # 3: tie hash
 # 4: dbm object
-# 5: configuration
 
-my ($FILENAME, $COLUMN_NAMES, $COLUMN_INDEX, $TIE_HASH, $DBM, $CONFIG) = (0 .. 5);
+my ($FILENAME, $COLUMN_NAMES, $COLUMN_INDEX, $TIE_HASH, $DBM) = (0 .. 4);
 
-sub config {
-	my ($self, $key, $value) = @_;
-	return $self->[$CONFIG]{$key} unless defined $value;
-	$self->[$CONFIG]{$key} = $value;
+sub create_table {
+    my ($class, $config, $filename, $columns) = @_;
+
+    return $class->create($columns, $filename, $config);
 }
 
 sub create {
-    my ($class, $config, $columns, $filename) = @_;
+    my ($class, $columns, $filename, $config) = @_;
 
     $config = {} unless defined $config;
     my ($File_permission_mode, $Fast_write)
@@ -81,13 +70,13 @@ sub create {
     $File_permission_mode = 0666 unless defined $File_permission_mode;
     $Fast_write = 1 unless defined $Fast_write;
 
-    die "columns argument $columns is not an array ref\n"
+    croak "columns argument $columns is not an array ref\n"
         unless ref($columns) eq 'ARRAY';
 
     # my $column_file = "$filename.columns";
     # my @columns = @$columns;
     # open(COLUMNS, ">$column_file")
-    #    or die "Couldn't create '$column_file': $!";
+    #    or croak "Couldn't create '$column_file': $!";
     # print COLUMNS join("\t", @columns), "\n";
     # close(COLUMNS);
 
@@ -101,11 +90,11 @@ sub create {
     my $flags = GDBM_NEWDB;
     $flags |= GDBM_FAST if $Fast_write;
     my $dbm = tie(%$tie, 'GDBM_File', $filename, $flags, $File_permission_mode)
-        or die "Could not create '$filename': $!";
+        or croak "Could not create '$filename': $!";
 
     $tie->{'c'} = join("\t", @$columns);
 
-    my $self = [$filename, $columns, $column_index, $tie, $dbm, $config];
+    my $self = [$filename, $columns, $column_index, $tie, $dbm];
     bless $self, $class;
 }
 
@@ -124,18 +113,8 @@ sub open_table {
         $flags |= GDBM_FAST if $Fast_write;
     }
 
-	my $dbm;
-    my $failed = 0;
-
-    while( $failed < 10 ) {
-        $dbm = tie(%$tie, 'GDBM_File', $filename, $flags, 0600)
-            and undef($failed), last;
-        $failed++;
-        select(undef,undef,undef,$failed * .100);
-    }
-
-    die("Could not tie to '$filename': $!")
-        if $failed;
+    my $dbm = tie(%$tie, 'GDBM_File', $filename, $flags, 0600)
+        or croak "Could not open '$filename': $!";
 
     my $columns = [split(/\t/, $tie->{'c'})];
 
@@ -145,15 +124,14 @@ sub open_table {
         $column_index->{$columns->[$i]} = $i;
     }
 
-    my $self = [$filename, $columns, $column_index, $tie, $dbm, $config];
+    my $self = [$filename, $columns, $column_index, $tie, $dbm];
     bless $self, $class;
 }
 
 sub close_table {
     my ($s) = @_;
-	splice(@$s, $DBM, 1);
-	my $ref = splice(@$s, $TIE_HASH, 1);
-    untie %$ref or die "Could not close GDBM table $s->[$FILENAME]: $!\n";
+
+    untie %{$s->[$TIE_HASH]} or die "Could not close '$s->[$FILENAME]': $!\n";
 }
 
 
@@ -171,31 +149,17 @@ sub test_column {
 sub column_index {
     my ($s, $column) = @_;
     my $i = $s->[$COLUMN_INDEX]{$column};
-    die "There is no column named '$column'" unless defined $i;
+    croak "There is no column named '$column'" unless defined $i;
     return $i;
 }
 
-sub row_hash {
-    my ($s, $key) = @_;
-	my %row;
-    @row{ @{$s->[$COLUMN_NAMES]} } = $s->row($key);
-	return \%row;
-}
-
-sub unstuff_row {
+sub row {
     my ($s, $key) = @_;
     my $line = $s->[$TIE_HASH]{"k$key"};
-    die "There is no row with index '$key'" unless defined $line;
+    croak "There is no row with index '$key'" unless defined $line;
     return map(unstuff($_), split(/\t/, $line, 9999));
 }
 
-sub thaw_row {
-    my ($s, $key) = @_;
-    my $line = $s->[$TIE_HASH]{"k$key"};
-    die "There is no row with index '$key'" unless defined $line;
-    return @{ Storable::thaw($line) };
-}
-
 sub field_accessor {
     my ($self, $column) = @_;
     my $index = $self->column_index($column);
@@ -216,27 +180,12 @@ sub field_settor {
     };
 }
 
-sub stuff_row {
+sub set_row {
     my ($s, $key, @fields) = @_;
     my $line = join("\t", map(stuff($_), @fields));
     $s->[$TIE_HASH]{"k$key"} = $line;
 }
 
-sub freeze_row {
-    my ($s, $key, @fields) = @_;
-    $s->[$TIE_HASH]{"k$key"} = Storable::freeze(\@fields);
-}
-
-if($Storable) {
-	*set_row = \&freeze_row;
-	*row = \&thaw_row;
-}
-else {
-	*set_row = \&stuff_row;
-	*row = \&unstuff_row;
-}
-
-
 sub field {
     my ($s, $key, $column) = @_;
     return ($s->row($key))[$s->column_index($column)];
@@ -244,10 +193,6 @@ sub field {
 
 sub set_field {
     my ($s, $key, $column, $value) = @_;
-    if($s->[$CONFIG]{Read_only}) {
-		::logError("Attempt to set $s->[$CONFIG]{name}::${column}::$key in read-only table");
-		return undef;
-	}
     my @row = $s->row($key);
     $row[$s->column_index($column)] = $value;
     $s->set_row($key, @row);
@@ -257,10 +202,6 @@ sub set_field {
 sub inc_field {
     my ($s, $key, $column, $adder) = @_;
     my($value);
-    if($s->[$CONFIG]{Read_only}) {
-		::logError("Attempt to set $s->[$CONFIG]{name}::${column}::$key in read-only table");
-		return undef;
-	}
     my @row = $s->row($key);
     $value = $row[$s->column_index($column)] += $adder;
     $s->set_row($key, @row);
@@ -279,13 +220,13 @@ sub ref {
 
 sub each_record {
     my ($s) = @_;
-    my $key;
+    my ($key, $value);
 
     for (;;) {
-        $key = each %{$s->[3]};
+        ($key, $value) = each %{$s->[3]};
         if (defined $key) {
             if ($key =~ s/^k//) {
-                return ($key, $s->row($key));
+                return ($key, map(unstuff($_), split(/\t/, $value, 9999)));
             }
         }
         else {
@@ -302,14 +243,8 @@ sub record_exists {
     return $r;
 }
 
-*test_record = \&record_exists;
-
 sub delete_record {
     my ($s, $key) = @_;
-    if($s->[$CONFIG]{Read_only}) {
-		::logError("Attempt to delete row '$key' in read-only table $s->[$CONFIG]{name}");
-		return undef;
-	}
 
     delete $s->[$TIE_HASH]{"k$key"};
 	1;
