@@ -1,9 +1,8 @@
 # Table/InMemory.pm: store a table in memory
 #
-# $Id: InMemory.pm,v 1.16 1999/06/07 08:08:31 mike Exp mike $
+# $Id: InMemory.pm,v 1.8 1997/05/22 07:10:45 mike Exp $
 #
 # Copyright 1995 by Andrew M. Wilcox <awilcox@world.std.com>
-# Copyright 1996-1998 by Mike Heins <mikeh@minivend.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,24 +19,12 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 package Vend::Table::InMemory;
-$VERSION = substr(q$Revision: 1.16 $, 10);
+$VERSION = substr(q$Revision: 1.8 $, 10);
+use Carp;
 use strict;
 
-# 0: column names
-# 1: column index
-# 2: tie hash
-# 3: configuration
-
-my ($COLUMN_NAMES, $COLUMN_INDEX, $TIE_HASH, $CONFIG) = (0 .. 3);
-
-sub config {
-	my ($self, $key, $value) = @_;
-	return $self->[$CONFIG]{$key} unless defined $value;
-	$self->[$CONFIG]{$key} = $value;
-}
-
-sub create {
-    my ($class, $config, $columns) = @_;
+sub create_table {
+    my ($class, $columns) = @_;
 
     my $column_index = {};
     my $i;
@@ -46,7 +33,7 @@ sub create {
     }
 
     my $hash = {};
-    my $self = [$columns, $column_index, $hash, $config];
+    my $self = [$columns, $column_index, $hash];
     bless $self, $class;
 }
 
@@ -69,36 +56,24 @@ sub test_column {
 sub column_index {
     my ($s, $column) = @_;
     my $i = $s->[1]{$column};
-    die "There is no column named '$column'" unless defined $i;
-    return $i;
+    croak "There is no column named '$column'" unless defined $i;
+    $i;
 }
 
-sub row_array {
-    my ($s, $key) = @_;
-    my $a = $s->[2]{$key};
-    die "There is no row with index '$key'" unless defined $a;
-    return @$a;
-}
-
-
 sub row {
     my ($s, $key) = @_;
     my $a = $s->[2]{$key};
-    die "There is no row with index '$key'" unless defined $a;
-	my %row;
-	@row{ @{$s->[0]} } = @$a;
-    return \%row;
+    croak "There is no row with index '$key'" unless defined $a;
+    return @$a;
 }
 
-*row_hash = \&row;
-
 sub field_accessor {
     my ($s, $column) = @_;
     my $index = $s->column_index($column);
     return sub {
         my ($key) = @_;
         my $a = $s->[2]{$key};
-        die "There is no row with index '$key'" unless defined $a;
+        croak "There is no row with index '$key'" unless defined $a;
         return $a->[$index];
     };
 }
@@ -123,7 +98,7 @@ sub set_row {
 sub field {
     my ($s, $key, $column) = @_;
     my $a = $s->[2]{$key};
-    die "There is no row with index '$key'" unless defined $a;
+    croak "There is no row with index '$key'" unless defined $a;
     return $a->[$s->column_index($column)];
 }
 
@@ -157,8 +132,6 @@ sub record_exists {
     return exists($s->[2]{$key});
 }
 
-*test_record = \&record_exists;
-
 sub delete_record {
     my ($s, $key) = @_;
     delete($s->[2]{$key});
