@@ -1,11 +1,9 @@
 # Vend/TextSearch.pm:  Search indexes with Perl
 #
-# $Id: TextSearch.pm,v 1.24 1999/06/07 08:07:53 mike Exp mike $
+# $Id: TextSearch.pm,v 1.8 1997/05/22 07:00:05 mike Exp $
 #
 # ADAPTED FOR USE WITH MINIVEND from Search::TextSearch
 #
-# Copyright 1996-1999 by Michael J. Heins <mikeh@iac.net>
-#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -20,16 +18,36 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
+
+#
+##  vi regex to delete debugs     %s/^\(.*$s->debug\)/#\1/
+##  vi regex to restore debugs     %s/^\([^#]*\)#\(.*$s->debug\)/\1\2/
+#
+
 package Vend::TextSearch;
 require Vend::Search;
 
+# AUTOLOAD
+#use AutoLoader;
+#@ISA = qw(Vend::Search);
+#sub import {}
+# END AUTOLOAD
+
+# NOAUTO
 @ISA = qw(Vend::Search);
+# END NOAUTO
 
-$VERSION = substr(q$Revision: 1.24 $, 10);
+$VERSION = substr(q$Revision: 1.8 $, 10);
 
+use Text::ParseWords;
 use Search::Dict;
 use strict;
 
+# AUTOLOAD
+#1;
+#__END__
+# END AUTOLOAD
+
 sub new {
     my ($class, %options) = @_;
 	my $self = new Vend::Search;
@@ -46,6 +64,8 @@ sub init {
 	my $g = $s->{global};
 	$g->{'dict_look'}	= '';
 	$g->{'dict_end'}	= '';
+	$g->{'dict_order'}	= 0;
+	$g->{'dict_case'} 	= 0;
 }
 
 sub version {
@@ -55,7 +75,7 @@ sub version {
 sub escape {
     my($self, @text) = @_;
     if($self->{'global'}->{all_chars}) {
-		@text = map {quotemeta $_} @text;
+		@text = grep quotemeta $_, @text;
     }
     for(@text) {
         s!([^\\]?)([}])!$1\\$2!g;
@@ -68,16 +88,12 @@ sub search {
 	my($s,%options) = @_;
     my $g = $s->{global};
 
-# DEBUG
-Vend::Util::logDebug
-("Text search using Vend::TextSearch\n")
-	if ::debug(0x10);
-# END DEBUG
+#	$s->debug("Vend::TextSearch");
 
-	my($delim);
+	my($delim,$string);
 	my($max_matches,$mod,$spec);
 	my($code,$count,$matches_to_send,@out);
-	my($index_delim,$limit_sub,$return_sub,$delayed_return);
+	my($index_delim,$limit_sub,$return_sub);
 	my($dict_limit,$f,$key,$val,$range_op);
 	my($return_file_name,$searchfile,@searchfiles);
 	my(@specs);
@@ -99,26 +115,10 @@ Vend::Util::logDebug
 		$g->{matches} = -1;
 		return undef; # If it makes it this far
 	}
+	@searchfiles = grep s!^([^/])!$g->{base_directory}/$1!, @searchfiles
+			if $g->{base_directory};
 
-	if($g->{base_directory}) {
-		for(@searchfiles) {
-			$_ = Vend::Util::catfile($g->{base_directory}, $_)
-				unless Vend::Util::file_name_is_absolute($_);
-		}
-	}
-
-	# Auto-index search
-	if($g->{dict_look} and $g->{dict_limit} =~ /[^-0-9]/) {
-		my $f = $g->{dict_limit};
-		$g->{dict_limit} = -1;
-		for (@searchfiles) {
-			next unless -f "$_.$f"; 
-			$_ .= ".$f";
-			$g->{return_fields} = [1];
-		}
-	}
-
- 	$return_file_name = $g->{return_file_name} || undef;
+ 	$return_file_name = $g->{return_file_name};
  	$index_delim = $g->{index_delim};
 	$g->{return_delim} = $index_delim
 		unless defined $g->{return_delim};
@@ -146,16 +146,70 @@ Vend::Util::logDebug
 
 	@specs = '' if @specs == 0;
 
-	@pats = $s->spec_check($g, @specs);
+#	$s->debug($s->dump_options() . "\n");
 
-# DEBUG
-Vend::Util::logDebug
-($s->dump_options() )
-	if ::debug(0x10);
-Vend::Util::logError
-($s->dump_options() . "specs: " . join('|', @pats) )
-    if $CGI::values{mv_search_debug};
-# END DEBUG
+  SPEC_CHECK: {
+	last SPEC_CHECK if $g->{return_all};
+	# Patch supplied by Don Grodecki
+	# Now ignores empty search strings if coordinated search
+	my $i = 0;
+
+	unless ($g->{coordinate} and @specs == @{$s->{fields}}) {
+		for(qw! case_sensitive substring_match negate !) {
+			next unless ref $g->{$_};
+			$g->{$_} = $g->{$_}->[0];
+		}
+		$g->{coordinate} = '';
+	}
+
+	while ($i < @specs) {
+		$string = $specs[$i];
+		if($#specs and length($string) == 0) { # should add a switch
+			if($g->{coordinate}) {
+		        splice(@{$s->{fields}}, $i, 1);
+		        splice(@{$g->{case_sensitive}}, $i, 1)
+					if ref $g->{case_sensitive};
+		        splice(@{$g->{substring_match}}, $i, 1)
+					if ref $g->{substring_match};
+		        splice(@{$g->{negate}}, $i, 1)
+					if ref $g->{negate};
+			}
+		    splice(@specs, $i, 1);
+			splice(@{$s->{specs}}, $i, 1);
+		}
+		elsif(length($string) < $g->{min_string}) {
+			my $msg = <<EOF;
+Search strings must be at least $g->{min_string} characters.
+You had '$string' as one of your search strings.
+EOF
+			&{$g->{error_routine}}($g->{error_page}, $msg);
+			$g->{matches} = -1;
+			return undef;
+		}
+		else {
+			$i++;
+		}
+	}
+
+#	$s->debug($s->dump_options() . "\n");
+
+	if ( ! $g->{exact_match} and ! $g->{coordinate}) {
+		@specs = $s->quoted_string( join ' ', @specs);
+	}
+
+	@specs = $s->escape(@specs);
+
+#	$s->debug("spec='" . (join "','", @specs) . "'");
+
+	# untaint
+	for(@specs) {
+		/(.*)/;
+		push @pats, $1;
+	}
+	@{$s->{'specs'}} = @specs;
+
+#	$s->debug("pats: '", join("', '", @pats), "'");
+  } # last SPEC_CHECK
 
 	if ($g->{coordinate}) {
 		undef $f;
@@ -181,23 +235,14 @@ Vend::Util::logError
 		return undef;
 	}
 
-	my $prospect;
-
-	eval {
-		($limit_sub, $prospect) = $s->get_limit($f);
-	};
-
+	eval {$limit_sub = $s->get_limit($f)};
 	if($@) {
 		&{$g->{error_routine}}($g->{error_page}, $@);
 		$g->{matches} = -1;
 		return undef;
 	}
 
-	$f = $prospect if $prospect;
-
-#::logGlobal("f=$f limit=$limit_sub");
-
-	eval {($return_sub, $delayed_return) = $s->get_return()};
+	eval {$return_sub = $s->get_return()};
 	if($@) {
 		&{$g->{error_routine}}($g->{error_page}, $@);
 		$g->{matches} = -1;
@@ -208,14 +253,7 @@ Vend::Util::logError
 
 	$g->{overflow} = 0;
 
-# DEBUG
-Vend::Util::logDebug
-('fields/specs: ' .  scalar @{$s->{fields}} . "/" .  scalar @{$s->{specs}} . "\n")
-	if ::debug(0x10);
-Vend::Util::logError
-('fields/specs: ' .  scalar @{$s->{fields}} . "/" .  scalar @{$s->{specs}} . "\n")
-    if $CGI::values{mv_search_debug};
-# END DEBUG
+#	$s->debug('fields/specs: ', scalar @{$s->{fields}}, " ", scalar @{$s->{specs}});
 
 	if($g->{dict_end}) {
 		if(!$g->{dict_order} && !$g->{dict_fold}) {
@@ -250,20 +288,18 @@ Vend::Util::logError
 
     my $sort_string = $s->find_sort();
 	# If the string is set, append the joined searcfiles;
-	if($sort_string and !$Global::Windows) {
+	if($sort_string) {
 		#$sort_string =~ s!\|\s*$!join ' ', @searchfiles, '|'!e;
 		@searchfiles = join ' ', 'cat', @searchfiles, '|', $sort_string;
 	}
-# DEBUG
-Vend::Util::logDebug
-("sort_string:  $sort_string\n")
-	if ::debug(0x10) ;
-# END DEBUG
+#	$s->debug("sort_string:  $sort_string");
 
 	local($/) = $g->{record_delim};
 
 	foreach $searchfile (@searchfiles) {
-		open(Vend::TextSearch::SEARCH, "< $searchfile")
+		$searchfile = "$g->{base_directory}/$searchfile"
+			unless ($sort_string || $searchfile =~ m:^/: || ! $g->{base_directory});
+		open(Vend::TextSearch::SEARCH, $searchfile)
 			or &{$g->{log_routine}}( "Couldn't open search file '$searchfile': $!"), next;
 		my $line;
 
@@ -272,161 +308,85 @@ Vend::Util::logDebug
         if($g->{head_skip} == 1) {
             my $field_names;
             chomp($field_names = <Vend::TextSearch::SEARCH>);
-            $g->{field_names} = [ split /\Q$index_delim/, $field_names]
+            $g->{field_names} = [ split /$index_delim/, $field_names]
                 unless defined $g->{field_names} || $sort_string;
         }
         elsif($g->{head_skip} > 1) {
-            while(<Vend::TextSearch::SEARCH>) {
+            while(<Search::TextSearch::SEARCH>) {
                 last if $. >= $g->{head_skip};
             }
         }
 
 		if($g->{dict_look}) {
-# DEBUG
-Vend::Util::logDebug
-("Dict search:  look='$g->{dict_look}'\n")
-	if ::debug(0x10);
-Vend::Util::logDebug
-("Dict search:   end='$g->{dict_end}'\n")
-	if ::debug(0x10);
-# END DEBUG
+#			$s->debug("Dict search:  look='$g->{dict_look}'");
+#			$s->debug("Dict search:   end='$g->{dict_end}'");
 			look \*Vend::TextSearch::SEARCH,
 				$g->{dict_look}, $g->{dict_order}, $g->{dict_fold};
 		}
 
 		if($g->{dict_end} && defined $limit_sub) {
-# DEBUG
-Vend::Util::logDebug
-("Dict search: with limit\n")
-	if ::debug(0x10);
-# END DEBUG
+#			$s->debug("Dict search: with limit");
 			while(<Vend::TextSearch::SEARCH>) {
 				last if &$dict_limit($_);
-# DEBUG
-Vend::Util::logDebug
-("Dict search: found='$_'\n")
-	if ::debug(0x10);
-# END DEBUG
-				next unless ! defined $f or &$f();
+#				#$s->debug("Dict search: found='$_'");
+				next unless &$f();
 				next unless &$limit_sub($_);
 				(push @out, $searchfile and last)
-					if defined $return_file_name;
+					if $return_file_name;
 				push @out, &$return_sub($_);
 			}
 		}
 		elsif($g->{dict_end}) {
-# DEBUG
-Vend::Util::logDebug
-("Dict search: NO limit\n")
-	if ::debug(0x10);
-# END DEBUG
+#			$s->debug("Dict search: NO limit");
 			while(<Vend::TextSearch::SEARCH>) {
 				last if &$dict_limit($_);
-# DEBUG
-Vend::Util::logDebug
-("Dict search: found='$_'\n")
-	if ::debug(0x10);
-# END DEBUG
+#				# #$s->debug("Dict search: found='$_'");
 				next unless &$f();
 				(push @out, $searchfile and last)
-					if defined $return_file_name;
+					if $return_file_name;
 				push @out, &$return_sub($_);
 			}
 		}
 		elsif(! defined $f and defined $limit_sub) {
-#::logGlobal("no f, limit");
 			while(<Vend::TextSearch::SEARCH>) {
 				next unless &$limit_sub($_);
 				(push @out, $searchfile and last)
-					if defined $return_file_name;
+					if $return_file_name;
 				push @out, &$return_sub($_);
 			}
 		}
 		elsif(defined $limit_sub) {
-#::logGlobal("f and limit");
 			while(<Vend::TextSearch::SEARCH>) {
 				next unless &$f();
-#::logGlobal("matched f");
 				next unless &$limit_sub($_);
 				(push @out, $searchfile and last)
-					if defined $return_file_name;
+					if $return_file_name;
 				push @out, &$return_sub($_);
 			}
 		}
-		elsif (!defined $f) {
-			&{$g->{error_routine}}($g->{error_page}, 'No search definition');
-			$g->{matches} = -1;
-			return undef;
-		}
 		else {
-#::logGlobal("just f");
 			while(<Vend::TextSearch::SEARCH>) {
 				next unless &$f();
 				(push @out, $searchfile and last)
-					if defined $return_file_name;
+					if $return_file_name;
 				push @out, &$return_sub($_);
 			}
 		}
 		close Vend::TextSearch::SEARCH;
 	}
 
-	if($delayed_return) {
-		$s->sort_search_return(\@out);
-		@out = map { &{$delayed_return}($_) } @out;
-	}
-
-	if($g->{unique_result}) {
-		my %seen;
-		@out = grep ! $seen{$_}++, @out;
-	}
-
 	$g->{matches} = scalar(@out);
+	$g->{first_match} = 0;
+
 
     if ($g->{matches} > $g->{match_limit}) {
         $s->save_more(\@out)
             or &{$g->{log_routine}}("Error saving matches: $!");
-		if ($g->{first_match}) {
-			splice(@out,0,$g->{first_match}) if $g->{first_match};
-			$g->{next_pointer} = $g->{first_match} + $g->{match_limit};
-			$g->{next_pointer} = 0
-				if $g->{next_pointer} > $g->{matches};
-		}
         $#out = $g->{match_limit} - 1;
     }
 
-# DEBUG
-Vend::Util::logDebug
-("$g->{matches} matches\n")
-	if ::debug(0x10);
-Vend::Util::logError
-("$g->{matches} matches")
-	if $CGI::values{mv_search_debug};
-Vend::Util::logDebug
-("0 .. " . (scalar(@out) - 1) . "\n" )
-	if ::debug(0x10);
-# END DEBUG
-
-	return \@out unless $g->{return_reference};
-
-	if($g->{return_reference} eq 'ARRAY') {
-		my $col = scalar @{$g->{return_fields}};
-		@out = map { [ split /$g->{return_delim}/, $_, $col ] } @out;
-	}
-	elsif($g->{return_format} eq 'HASH') {
-		my $col = scalar @{$g->{return_fields}};
-		my @col;
-		my @names;
-		@names = @{$g->{field_names}};
-		$names[0] eq '0' and $names[0] = 'code';
-		my %hash;
-		my $key;
-		for (@out) {
-			@col = split /$g->{return_delim}/, $_, $col;
-			$hash{$col[0]} = {};
-			@{ $hash{$col[0]} } {@names} = @col;
-		}
-		return \%hash;
-	}
+#	$s->debug($g->{matches}, " matches");
+#	$s->debug("0 .. ", (scalar(@out) - 1));
 
 	\@out;
 }
