@@ -1,4 +1,4 @@
-# $Id: ValidCC.pm,v 1.13 1999/06/07 08:08:09 mike Exp mike $
+# $Id: ValidCC.pm,v 1.4 1997/06/17 04:22:52 mike Exp $
 #
 # ValidCC.pm - validate credit card numbers
 #
@@ -7,11 +7,20 @@
 # Modified by Mike to make more forgiving in the parameters.
 
 package Vend::ValidCC;
-$VERSION = substr(q$Revision: 1.13 $, 10);
+$VERSION = substr(q$Revision: 1.4 $, 10);
 require 5.000;
 require Exporter;
+use Carp;
 
+# AUTOLOAD
+#use AutoLoader;
+#@ISA = qw(Exporter Autoloader);
+#*AUTOLOAD = \&AutoLoader::AUTOLOAD;
+# END AUTOLOAD
+
+# NOAUTO
 @ISA = qw(Exporter);
+# END NOAUTO
 
 @EXPORT = qw(ValidCreditCard encrypt_cc encrypt_standard_cc);
 
@@ -51,6 +60,11 @@ Bruce Albrecht (bruce@zuhause.mn.org)
 
 =cut
 
+# AUTOLOAD
+#1;
+#__END__
+# END AUTOLOAD
+
 
 sub ValidCreditCard
 {
@@ -129,11 +143,21 @@ sub encrypt_cc {
 	my $infile    = 0;
 
 	$cmd = $Vend::Cfg->{'EncryptProgram'};
-	$cmd = '' if "\L$cmd" eq 'none';
-	
+
+	# This is the internal function, will return the value
+	# only if it was found. Takes the IVEC from the first
+	# eight characters of the encrypted password
+	if (defined @Des::EXPORT and $cmd =~ /^internal/i ) {
+		my($password) = $Vend::Cfg->{'Password'} || return undef;
+		my($ivec) = $Vend::Cfg->{'Pw_Ivec'} || return undef;
+		my $key = string_to_key($password);
+		my $sched = set_key($key);
+		$encrypted = pcbc_encrypt($enclair,undef,$sched,$ivec);
+		return $encrypted;
+	}
 
 	#Substitute the password
-	unless (! $cmd or $cmd =~ /\bpgp\b/ or $cmd =~ s/%p/$password/ ) {
+	unless ($cmd =~ /\bpgp\b/ or $cmd =~ s/%p/$password/ ) {
 		$firstline = 1;
 	}
 
@@ -153,7 +177,6 @@ sub encrypt_cc {
 			die "Couldn't write $tempfile: $!\n";
 		# Put the cardnumber there, and maybe password first
 		print CARD "$password\n" if $firstline;
-		$enclair .= "\r\n\cZ\r\n" if $Global::Windows;
 		print CARD $enclair;
 		close CARD;
 
@@ -165,13 +188,12 @@ sub encrypt_cc {
 		$status = $?;
 	}
 	else {
-		$cmd = "| $cmd " if $cmd;
-		open(CRYPT, "$cmd>$tempfile ") || die "Couldn't fork: $!\n";
+		open(CRYPT, "| $cmd >$tempfile ") || die "Couldn't fork: $!\n";
 		print CRYPT $enclair;
 		close CRYPT;
-		$status = $cmd ? $? : 0;
+		$status = $?;
 
-		open(CARD, "< $tempfile") || warn "open $tempfile: $!\n";
+		open(CARD, $tempfile) || warn "open $tempfile: $!\n";
 		$encrypted = <CARD>;
 		close CARD;
 	}
@@ -206,22 +228,21 @@ sub encrypt_standard_cc {
 	my $num		= $ref->{mv_credit_card_number}		|| '';
 	my $year	= $ref->{mv_credit_card_exp_year}	|| '';
 	my $all		= $ref->{mv_credit_card_exp_all}	|| '';
-	my $force	= $ref->{mv_credit_card_force}		|| '';
 
 	for ( qw (	mv_credit_card_type		mv_credit_card_number
 				mv_credit_card_exp_year	mv_credit_card_exp_month
-				mv_credit_card_exp_all  mv_credit_card_force))
+				mv_credit_card_exp_all  ))
 	{
 		next unless defined $ref->{$_};
 		delete $ref->{$_};
 	}
 
 	# remove unwanted chars from card number
-	$num =~ tr/0-9//cd;
+	#$num =~ tr/0-9//cd;
 
 	# error will be pushed on this if present
 	@return = (
-				'',			# 0- Whether it is valid
+				0,			# 0- Whether it is valid
 				'',			# 1- Encrypted credit card information
 				'',			# 2- Month
 				'',			# 3- Year
@@ -270,14 +291,14 @@ sub encrypt_standard_cc {
 
 	#$num =~ tr/\d//cd;
 
-	unless ($valid = ValidCreditCard ($type,$num,$all) or $force ) {
+	unless ($valid = ValidCreditCard ($type,$num,$all) ) {
 		push @return, "Credit card number, type, or expiration not valid.";
 		return @return;
 	}
 
 	$return[0] = $valid;
 
-	$info = encrypt_cc "$type\t$num\t$all\n";
+	$info = encrypt_cc "$type $num $all\n";
 
 	unless (defined $info) {
 		push @return, "Credit card encryption failed.";
